// tcp_client1.cpp
// Glenn G. Chappell
// 12 Apr 2013
//
// For CS 321 Spring 2013
// HTTP Client with Error Checking

#include "osl/socket.h"
// TO RUN THIS UNDER NETRUN, uncomment the following line
//#include "osl/socket.cpp"

#include <iostream>
using std::cout;
using std::endl;
using std::cerr;
#include <string>
using std::string;
#include <sstream>
using std::istringstream;
#include <cstdlib>
using std::exit;


// my_skt_abort
// Substitute for Lawlor's default skt_abort function
// Prints err msg to stderr & returns -1
// Does NOT exit
int my_skt_abort(int code,const char *msg)
{
    cerr << "Fatal socket error-- "
         << msg << " (" << code << ")" << endl;
    return -1;
}


// my_skt_recv_string
// Substitute for Lawlor's skt_recv_string
// Does error checking
// Returns 0 on success, -1 on error, using skt_abort
// String read is in theString
int my_skt_recv_string(std::string & theString,
                       SOCKET skt,
                       const char *term=" \t\r\n")
{
    char c; 
    theString = "";
    while (true) {
        int retval = skt_recvN(skt, &c, 1); // Grab next character
        if (retval != 0) return retval;
        if (strchr(term, c))
        {
            if (c=='\r') continue;  // will be CR/LF; wait for LF
            else return 0;          // Hit terminator-- stop.
        }
        else theString += c; // normal char; add to string, continue
    }
}


int main()
{
    int retval;              // For return values of skt funcs
    string buffer;           // Holds chars received over socket

    // Set skt abort function to mine (above)
    skt_set_abort(my_skt_abort);

    unsigned int port = 80;  // Standard HTTP port
    //skt_ip_t ip = { 137, 229, 25, 100 };
                             // Use this form to specify the IP no.
    skt_ip_t ip = skt_lookup_ip("www.cs.uaf.edu");
    if (skt_ip_match(ip, _skt_invalid_ip))
    {
        cerr << "EXITING" << endl;
        exit(1);
    }

    // Try to connect with the give IP & port number
    SOCKET s = skt_connect(ip, port, 5);
                             // 5-second timeout
    if (s == -1)
    {
        cerr << "EXITING" << endl;
        exit(1);
    }

    string msg =
        "GET /2013/spring/cs321/ HTTP/1.1\r\n"
        "Host: www.cs.uaf.edu\r\n"
        "\r\n";              // HTTP request to send

    cout << "Sending HTTP request" << endl;

    // Send message
    retval = skt_sendN(s, msg.c_str(), msg.size());
    if (retval == -1)
    {
        cerr << "EXITING" << endl;
        exit(1);
    }

    cout << "Printing response" << endl << endl;

    // Read header of response
    int conlen = 0;          // Holds content length
    do
    {
        retval = my_skt_recv_string(buffer, s, "\r\n");
        if (retval == -1)
        {
            cerr << "EXITING" << endl;
            exit(1);
        }
        cout << "Header line: " << buffer << endl;
        if (buffer.substr(0, 15) == "Content-Length:")
        {
            istringstream is(buffer.substr(15));
            is >> conlen;
        }
    }
    while (!buffer.empty()); // Header ends with blank line

    cout << endl;
    cout << "Content length = " << conlen << endl;
    cout << endl;

    // Read body of response, if any, and print it
    if (conlen == 0)
    {
        cout << "Body is empty" << endl;
    }
    else
    {
        buffer.resize(conlen, '?');
        retval = skt_recvN(s, &buffer[0], conlen);
        if (retval == -1)
        {
            cerr << "EXITING" << endl;
            exit(1);
        }
        cout << "Body:" << endl;

        // To print the whole body, replace the 5 lines below
        //  with the commented-out line.
        cout << buffer.substr(0,100) << endl;
        cout << endl;
        cout << "NOTE: To avoid filling up your screen, I have only\n";
        cout << "printed the first 100 characters of the body. You\n";
        cout << "can change this if you wish; see the source code.\n";
        //cout << buffer << endl;
    }

    // Done; close socket
    skt_close(s);

    return 0;
}

